searchData={"items":[{"type":"module","title":"Permit.Phoenix","doc":"Phoenix, Plug and LiveView integrations integration for [Permit](https://github.com/curiosum-dev/permit).\n\nIn Phoenix controller actions as well as LiveView modules, based on the resource configured for the current controller or LiveView (e.g. an `Article`), the current user, and the action (determined by the controller action or `:live_action`), it typically performs the following operations:application\n\n1. Based on current params, typically an ID parameter, preload a record which is the subject of the current action. This is done either from the DB when `Permit.Ecto` is used or in a different manner.\n\n2. Call to the application's authorization module (the one that has `use Permit` - refer to [docs for `Permit`](https://hexdocs.pm/permit/Permit.html#module-check-a-user-s-authorization-to-perform-an-action-on-a-resource) for examples) to ask whether the preloaded record is authorized to perform the current controller action.\n\n3. Expose the loaded record into `assigns` if authorized, or perform a defined action (e.g. redirect) otherwise.\n\nTherefore, whereas in plain Phoenix the following could be written:\n```elixir\nimport MyApp.Authorization\n\ndef show(conn, %{\"id\" => id} = params) do\n  article = MyApp.Repo.get(MyApp.Article, id)\n  user = conn.assigns.current_user\n\n  if can(user) |> read?(article) do\n    conn\n    |> render(:show, loaded_resource: article)\n  else\n    conn\n    |> put_flash(:error, \"You do not have permission to perform this action.\")\n    |> redirect(to: \"/\")\n  end\nend\n```\n\nWhen using `Permit.Phoenix`, it becomes:\n```elixir\nuse Permit.Phoenix.Controller,\n  authorization_module: MyApp.Authorization,\n  resource_module: MyApp.Article\n\ndef show(conn, params) do\n  render(conn, :show)\nend\n```\n\nThat's it - the preload, the current user fetching, the assignment of loaded record and the handling of authorization errors is automatic.\n\nThe way authorization errors are handled, the manner of loading records from the Ecto Repo or elsewhere, and many other parameters, are customizable - see `Permit.Phoenix.Controller` and `Permit.Phoenix.LiveView` for more information.","ref":"Permit.Phoenix.html"},{"type":"module","title":"Permit.Phoenix.Actions","doc":"Provides default action groupings and singular actions for Phoenix controllers and live views.\nThis module is used as a default implementation that can be overridden in individual modules.\n\n\nAlso, allows inferring actions from a Phoenix router module for convenience, so that the actions module\ndoes not need to repeat action names already living in controllers.\n\nExample:\n\n    defmodule MyApp.Router do\n      # ...\n\n      get(\"/items/:id\", MyApp.ItemController, :view)\n    end\n\n    defmodule MyApp.Actions do\n      # Merge the actions from the router into the default grouping schema.\n      use Permit.Phoenix.Actions, router: MyApp.Router\n    end\n\n    defmodule MyApp.Permissions do\n      # Use the actions module to define permissions.\n      use Permit.Permissions, actions_module: MyApp.Actions\n\n      def can(:admin = _role) do\n        permit()\n        |> all(Item)\n      end\n\n      # The `view` action is automatically added to the grouping schema\n      # and hence available as a `view/2`function when defining permissions.\n      def can(:owner = _role) do\n        permit()\n        |> view(Item)\n        |> all(Item, fn user, item -> item.owner_id == user.id end)\n      end\n\n    end","ref":"Permit.Phoenix.Actions.html"},{"type":"function","title":"Permit.Phoenix.Actions.actions_from_router/1","doc":"","ref":"Permit.Phoenix.Actions.html#actions_from_router/1"},{"type":"function","title":"Permit.Phoenix.Actions.crud_grouping/0","doc":"Convenience function defining the basic CRUD (create, read, update, delete) actions.","ref":"Permit.Phoenix.Actions.html#crud_grouping/0"},{"type":"function","title":"Permit.Phoenix.Actions.crud_singular/0","doc":"Convenience function returning actions that are singular in the most basic CRUD setup, in which case\nall of: `:create`, `:read`, `:update` and `:delete` are singular.","ref":"Permit.Phoenix.Actions.html#crud_singular/0"},{"type":"function","title":"Permit.Phoenix.Actions.grouping_schema/0","doc":"Returns the default action grouping schema for Phoenix applications.","ref":"Permit.Phoenix.Actions.html#grouping_schema/0"},{"type":"function","title":"Permit.Phoenix.Actions.merge_from_router/2","doc":"","ref":"Permit.Phoenix.Actions.html#merge_from_router/2"},{"type":"function","title":"Permit.Phoenix.Actions.singular_actions/0","doc":"Returns the list of actions that operate on a single resource.","ref":"Permit.Phoenix.Actions.html#singular_actions/0"},{"type":"behaviour","title":"Permit.Phoenix.Controller","doc":"Injects authorization plug (Permit.Phoenix.Plug), allowing to\nprovide its options either directly in options of `use`, or\nas overridable functions.\n\nExample:\n\n    # my_app_web.ex\n    def controller do\n      use Permit.Phoenix.Controller,\n        authorization_module: MyApp.Authorization,\n        fallback_path: \"/unauthorized\"\n    end\n\n    # your controller module\n    defmodule MyAppWeb.PageController do\n      use MyAppWeb, :live_view\n\n      @impl true\n      def resource_module, do: MyApp.Item\n\n      # Override default action groupings if needed\n      @impl true\n      def action_grouping do\n        %{\n          new: [:create],\n          index: [:read],\n          show: [:read],\n          edit: [:update],\n          create: [:create],\n          update: [:update],\n          delete: [:delete]\n        }\n      end\n\n      # Override singular actions if needed\n      @impl true\n      def singular_actions do\n        [:show, :edit, :new, :delete, :update]\n      end\n    end","ref":"Permit.Phoenix.Controller.html"},{"type":"callback","title":"Permit.Phoenix.Controller.action_grouping/0","doc":"Defines the action grouping schema for this controller.\nThis can be overridden in individual controllers to customize the action mapping.","ref":"Permit.Phoenix.Controller.html#c:action_grouping/0"},{"type":"callback","title":"Permit.Phoenix.Controller.authorization_module/0","doc":"Configures the controller with the application's authorization configuration.","ref":"Permit.Phoenix.Controller.html#c:authorization_module/0"},{"type":"callback","title":"Example - Permit.Phoenix.Controller.authorization_module/0","doc":"@impl Permit.Phoenix.Controller\n    def authorization_module, do: MyApp.Authorization\n\n    # Requires defining an authorization configuration module\n    defmodule MyApp.Authorization, do:\n      use Permit, permissions_module: MyApp.Permissions","ref":"Permit.Phoenix.Controller.html#c:authorization_module/0-example"},{"type":"callback","title":"Permit.Phoenix.Controller.base_query/1","doc":"Creates the basis for an Ecto query constructed by `Permit.Ecto` based on controller action, resource module, subject (typically `:current_user`) and controller params.\n\nTypically useful when using [nested resource routes](https://hexdocs.pm/phoenix/routing.html#nested-resources). In an action routed like `/users/:user_id/posts/:id`, you can use the `c:base_query/1` callback to filter records by `user_id`, while filtering by `id` itself will be applied automatically (the name of the ID parameter can be overridden with the `c:id_`).","ref":"Permit.Phoenix.Controller.html#c:base_query/1"},{"type":"callback","title":"Example - Permit.Phoenix.Controller.base_query/1","doc":"defmodule MyApp.CommentController do\n      use Permit.Phoenix.Controller,\n        authorization_module: MyApp.Authorization\n        resource_module: MyApp.Blog.Comment\n\n      @impl true\n      def base_query(%{\n        action: :index,\n        params: %{\"article_id\" => article_id}\n      }) do\n        MyApp.CommentQueries.by_article_id(article_id)\n      end\n    end","ref":"Permit.Phoenix.Controller.html#c:base_query/1-example"},{"type":"callback","title":"Permit.Phoenix.Controller.except/0","doc":"Allows opting out of using Permit for given controller actions.\n\nDefaults to `[]`, thus by default all actions are guarded with Permit.","ref":"Permit.Phoenix.Controller.html#c:except/0"},{"type":"callback","title":"Permit.Phoenix.Controller.fallback_path/2","doc":"If `c:handle_unauthorized/2` is not defined, sets the fallback path to which the user is redirected on authorization failure.\n\nDefaults to `/`.","ref":"Permit.Phoenix.Controller.html#c:fallback_path/2"},{"type":"callback","title":"Permit.Phoenix.Controller.fetch_subject/1","doc":"Retrieves the current user from `conn` as the authorization subject. Defaults to `conn.assigns[:current_user]`.","ref":"Permit.Phoenix.Controller.html#c:fetch_subject/1"},{"type":"callback","title":"Example - Permit.Phoenix.Controller.fetch_subject/1","doc":"@impl true\n    def fetch_subject(%{assigns: assigns}) do\n      assigns[:user]\n    end","ref":"Permit.Phoenix.Controller.html#c:fetch_subject/1-example"},{"type":"callback","title":"Permit.Phoenix.Controller.finalize_query/2","doc":"Post-processes an Ecto query constructed by `Permit.Ecto` based on controller action, resource module, subject (typically `:current_user`) and controller params.\n\nTypically useful when using [nested resource routes](https://hexdocs.pm/phoenix/routing.html#nested-resources). In an action routed like `/users/:user_id/posts/:id`, you can use the `c:base_query/1` callback to filter records by `user_id`, while filtering by `id` itself will be applied automatically (the name of the ID parameter can be overridden with the `c:id_`).","ref":"Permit.Phoenix.Controller.html#c:finalize_query/2"},{"type":"callback","title":"Example - Permit.Phoenix.Controller.finalize_query/2","doc":"defmodule MyApp.CommentController do\n      use Permit.Phoenix.Controller,\n        authorization_module: MyApp.Authorization\n        resource_module: MyApp.Blog.Comment\n\n      # just for demonstration - please don't do it directly in controllers\n      import Ecto.Query\n\n      @impl true\n      def finalize_query(query, %{\n        action: :index,\n      }) do\n        query\n        |> preload([c], [:user])\n      end\n    end","ref":"Permit.Phoenix.Controller.html#c:finalize_query/2-example"},{"type":"callback","title":"Permit.Phoenix.Controller.handle_not_found/1","doc":"","ref":"Permit.Phoenix.Controller.html#c:handle_not_found/1"},{"type":"function","title":"Permit.Phoenix.Controller.handle_not_found/2","doc":"","ref":"Permit.Phoenix.Controller.html#handle_not_found/2"},{"type":"callback","title":"Permit.Phoenix.Controller.handle_unauthorized/2","doc":"Called when authorization on an action or a loaded record is not granted. Must halt `conn` after rendering or redirecting.","ref":"Permit.Phoenix.Controller.html#c:handle_unauthorized/2"},{"type":"callback","title":"Example - Permit.Phoenix.Controller.handle_unauthorized/2","doc":"defmodule MyApp.CommentController do\n      use Permit.Phoenix.Controller,\n        authorization_module: MyApp.Authorization\n        resource_module: MyApp.Blog.Comment\n\n      @impl true\n      def handle_unauthorized(action, conn) do\n        case get_format(conn) do\n          \"json\" ->\n            # render a 4xx JSON response\n\n          \"html\" ->\n            # handle HTML response, e.g. redirect\n        end\n      end\n    end","ref":"Permit.Phoenix.Controller.html#c:handle_unauthorized/2-example"},{"type":"callback","title":"Permit.Phoenix.Controller.id_param_name/2","doc":"Sets the name of the ID param that will be used for preloading a record for authorization.\n\nDefaults to `\"id\"`. If the route contains a different name of the record ID param, it should be changed accordingly.","ref":"Permit.Phoenix.Controller.html#c:id_param_name/2"},{"type":"callback","title":"Permit.Phoenix.Controller.id_struct_field_name/2","doc":"Sets the name of the field that contains the resource's ID which should be looked for.\n\nDefaults to `:id`. If the record's ID (usually a primary key) is in a different field, then it should be changed accordingly.","ref":"Permit.Phoenix.Controller.html#c:id_struct_field_name/2"},{"type":"callback","title":"Permit.Phoenix.Controller.loader/1","doc":"If `Permit.Ecto` is not used, it allows defining a loader function that loads a record or a list of records, depending on action type (singular or plural).","ref":"Permit.Phoenix.Controller.html#c:loader/1"},{"type":"callback","title":"Example - Permit.Phoenix.Controller.loader/1","doc":"@impl true\n    def loader(%{action: :index, params: %{page: page}}),\n      do: ItemContext.load_all(page: page)\n\n    def loader(%{action: :show}, params: %{id: id}),\n      do: ItemContext.load(id)","ref":"Permit.Phoenix.Controller.html#c:loader/1-example"},{"type":"callback","title":"Permit.Phoenix.Controller.preload_actions/0","doc":"Declares which actions in the controller are to use Permit's automatic preloading and authorization in addition to defaults: `[:show, :edit, :update, :delete, :index]`.\n\nDefaults to `[]`, which means that `[:show, :edit, :update, :delete, :index]` and no other actions will use preloading.","ref":"Permit.Phoenix.Controller.html#c:preload_actions/0"},{"type":"callback","title":"Permit.Phoenix.Controller.resource_module/0","doc":"Declares the controller's resource module. For instance, when Phoenix and Ecto is used, typically for an `ArticleController` the resource will be an `Article` Ecto schema.\n\nThis resource module, along with the controller action name, will be used for authorization checks before each action.\n\nIf `Permit.Ecto` is used, this setting selects the Ecto schema which will be used for automatic preloading a record for authorization.","ref":"Permit.Phoenix.Controller.html#c:resource_module/0"},{"type":"callback","title":"Example - Permit.Phoenix.Controller.resource_module/0","doc":"defmodule MyApp.ArticleController do\n      use Permit.Phoenix.Controller\n\n      def authorization_module, do: MyApp.Authorization\n\n      def resource_module, do: MyApp.Article\n\n      # Alternatively, you can do the following:\n\n      use Permit.Phoenix.Controller,\n        authorization_module: MyApp.Authorization,\n        resource_module: MyApp.Blog.Article\n    end","ref":"Permit.Phoenix.Controller.html#c:resource_module/0-example"},{"type":"callback","title":"Permit.Phoenix.Controller.singular_actions/0","doc":"Defines which actions are considered singular (operating on a single resource).\nThis can be overridden in individual controllers to customize the singular actions.","ref":"Permit.Phoenix.Controller.html#c:singular_actions/0"},{"type":"callback","title":"Permit.Phoenix.Controller.unauthorized_message/2","doc":"","ref":"Permit.Phoenix.Controller.html#c:unauthorized_message/2"},{"type":"function","title":"Permit.Phoenix.Controller.unauthorized_message/3","doc":"","ref":"Permit.Phoenix.Controller.html#unauthorized_message/3"},{"type":"behaviour","title":"Permit.Phoenix.LiveView","doc":"A live view module using the authorization mechanism should mix in the LiveViewAuthorization\nmodule:\n\n    defmodule MyAppWeb.DocumentLive.Index\n      use Permit.Phoenix.LiveView\n\n      @impl true\n      def resource_module, do: Document\n\n      # Override default action groupings if needed\n      @impl true\n      def action_grouping do\n        %{\n          new: [:create],\n          index: [:read],\n          show: [:read],\n          edit: [:update],\n          create: [:create],\n          update: [:update],\n          delete: [:delete]\n        }\n      end\n\n      # Override singular actions if needed\n      @impl true\n      def singular_actions do\n        [:show, :edit, :new, :delete, :update]\n      end\n    end\n\nwhich adds the LiveViewAuthorization behavior with the following callbacks to be implemented -\nfor example:\n\n    # The related schema\n\n    @impl true\n    def resource_module, do: Document\n\n    # Loader function for a singular resource in appropriate actions (:show, etc.); usually a context\n    # function. If not defined, Repo.get is used by default.\n\n    @impl true\n    def loader, do: fn id -> get_organization!(id) end\n\n    # How to fetch the current user from session - for instance:\n\n    @impl true\n    def fetch_subject(socket, session) do\n      with token when not is_nil(token) <- session[\"token\"],\n           %User{} = current_user <- get_user(token) do\n        current_user\n      else\n        _ -> nil\n      end\n    end\n\nOptionally, p handle_unauthorized/2 optional callback can be implemented, returning {:cont, socket}\nor {:halt, socket}. The default implementation returns:\n\n    {:halt, socket(socket, to: socket.view.fallback_path())}","ref":"Permit.Phoenix.LiveView.html"},{"type":"callback","title":"Permit.Phoenix.LiveView.action_grouping/0","doc":"Defines the action grouping schema for this live view.\nThis can be overridden in individual live views to customize the action mapping.","ref":"Permit.Phoenix.LiveView.html#c:action_grouping/0"},{"type":"callback","title":"Permit.Phoenix.LiveView.authorization_module/0","doc":"","ref":"Permit.Phoenix.LiveView.html#c:authorization_module/0"},{"type":"callback","title":"Permit.Phoenix.LiveView.base_query/1","doc":"","ref":"Permit.Phoenix.LiveView.html#c:base_query/1"},{"type":"callback","title":"Permit.Phoenix.LiveView.event_mapping/0","doc":"","ref":"Permit.Phoenix.LiveView.html#c:event_mapping/0"},{"type":"callback","title":"Permit.Phoenix.LiveView.except/0","doc":"","ref":"Permit.Phoenix.LiveView.html#c:except/0"},{"type":"callback","title":"Permit.Phoenix.LiveView.fallback_path/2","doc":"","ref":"Permit.Phoenix.LiveView.html#c:fallback_path/2"},{"type":"callback","title":"Permit.Phoenix.LiveView.fetch_subject/2","doc":"","ref":"Permit.Phoenix.LiveView.html#c:fetch_subject/2"},{"type":"callback","title":"Permit.Phoenix.LiveView.finalize_query/2","doc":"","ref":"Permit.Phoenix.LiveView.html#c:finalize_query/2"},{"type":"callback","title":"Permit.Phoenix.LiveView.handle_not_found/1","doc":"","ref":"Permit.Phoenix.LiveView.html#c:handle_not_found/1"},{"type":"function","title":"Permit.Phoenix.LiveView.handle_not_found/2","doc":"","ref":"Permit.Phoenix.LiveView.html#handle_not_found/2"},{"type":"callback","title":"Permit.Phoenix.LiveView.handle_unauthorized/2","doc":"","ref":"Permit.Phoenix.LiveView.html#c:handle_unauthorized/2"},{"type":"callback","title":"Permit.Phoenix.LiveView.id_param_name/2","doc":"","ref":"Permit.Phoenix.LiveView.html#c:id_param_name/2"},{"type":"callback","title":"Permit.Phoenix.LiveView.id_struct_field_name/2","doc":"","ref":"Permit.Phoenix.LiveView.html#c:id_struct_field_name/2"},{"type":"callback","title":"Permit.Phoenix.LiveView.loader/1","doc":"","ref":"Permit.Phoenix.LiveView.html#c:loader/1"},{"type":"function","title":"Permit.Phoenix.LiveView.mounting?/1","doc":"Returns true if inside mount/1, false otherwise. Useful for distinguishing between\nrendering directly via router or being in a handle_params lifecycle.\n\nFor example, a handle_unauthorized/1 implementation must redirect when halting during mounting,\nwhile it needn't redirect when halting during the handle_params lifecycle.","ref":"Permit.Phoenix.LiveView.html#mounting?/1"},{"type":"function","title":"Example - Permit.Phoenix.LiveView.mounting?/1","doc":"@impl true\n        def handle_unauthorized(socket) do\n          if mounting?(socket) do\n            {:halt, push_redirect(socket, to: \"/foo\")}\n          else\n            {:halt, assign(socket, :unauthorized, true)}\n          end\n        end","ref":"Permit.Phoenix.LiveView.html#mounting?/1-example"},{"type":"callback","title":"Permit.Phoenix.LiveView.preload_actions/0","doc":"","ref":"Permit.Phoenix.LiveView.html#c:preload_actions/0"},{"type":"callback","title":"Permit.Phoenix.LiveView.resource_module/0","doc":"","ref":"Permit.Phoenix.LiveView.html#c:resource_module/0"},{"type":"callback","title":"Permit.Phoenix.LiveView.singular_actions/0","doc":"Defines which actions are considered singular (operating on a single resource).\nThis can be overridden in individual live views to customize the singular actions.","ref":"Permit.Phoenix.LiveView.html#c:singular_actions/0"},{"type":"callback","title":"Permit.Phoenix.LiveView.unauthorized_message/2","doc":"","ref":"Permit.Phoenix.LiveView.html#c:unauthorized_message/2"},{"type":"function","title":"Permit.Phoenix.LiveView.unauthorized_message/3","doc":"","ref":"Permit.Phoenix.LiveView.html#unauthorized_message/3"},{"type":"callback","title":"Permit.Phoenix.LiveView.use_stream?/1","doc":"","ref":"Permit.Phoenix.LiveView.html#c:use_stream?/1"},{"type":"module","title":"Permit.Phoenix.LiveView.AuthorizeHook","doc":"Hooks into the :mount and :handle_params lifecycles to authorize the current action.\nThe current action is denoted by the :live_action assign (retrieved from the router),\nfor example with the following route definition:\n\n    live \"/organizations\", OrganizationLive.Index, :index\n\nthe :live_action assign value will be :index.","ref":"Permit.Phoenix.LiveView.AuthorizeHook.html"},{"type":"module","title":"Configuration - Permit.Phoenix.LiveView.AuthorizeHook","doc":"In the router, use the :on_mount option of live_session to configure it, passing a tuple.\nFor convenience, you might return this tuple from a function that you might import into the router.\n\n  live_session :some_session, on_mount: Permit.Phoenix.LiveView.AuthorizeHook do\n    live \"/organizations\", MyLive.Index, :index\n    # ...\n  end","ref":"Permit.Phoenix.LiveView.AuthorizeHook.html#module-configuration"},{"type":"module","title":"Usage - Permit.Phoenix.LiveView.AuthorizeHook","doc":"Authorization is done on live view mount and in handle_params (where the URL, and\nhence assigns.live_action, may change).\n\nA live view module using the authorization mechanism should mix in the LiveViewAuthorization\nmodule:\n\n    defmodule MyAppWeb.DocumentLive.Index\n      use Permit.Phoenix.LiveView\n    end\n\nwhich adds the LiveViewAuthorization behavior with the following callbacks to be implemented -\nfor example:\n\n    # The related schema\n    def resource_module, do: Document\n\n    # Loader function for a singular resource in appropriate actions (:show, etc.); usually a context\n    # function. If not defined, Repo.get is used by default.\n    def loader, do: fn id -> get_organization!(id) end\n\nDepending on whether you use `use MyAppWeb, :live_view` or not to configure your LiveViews,\nit might be more convenient to provide configuration as options when you mix it in via `use`.\nFor instance:\n\n    # my_app_web.ex\n    def live_view do\n      use Permit.Phoenix.LiveView,\n        authorization_module: Lvauth.Authorization,\n        fallback_path: \"/unauthorized\"\n    end\n\n    # your live view module\n    defmodule MyAppWeb.PageLive do\n      use MyAppWeb, :live_view\n\n      @impl true\n      def resource_module, do: MyApp.Item\n\n      # you might or might not want to override something here\n      @impl true\n      def fallback_path: \"/foo\"\n    end\n\nIn actions like :show, where a singular resource is to be authorized and preloaded, it is preloaded into\nthe :loaded_resources assign. This way, you can implement your :handle_params event without caring\nabout loading the record:\n\n    @impl true\n    def handle_params(_params, _, socket) do\n      {:noreply,\n      socket\n      |> assign(:page_title, page_title(socket.assigns.live_action))\n      |> assign(:organization, socket.assigns.loaded_resources)}\n    end\n\nOptionally, a handle_unauthorized/2 optional callback can be implemented, returning {:cont, socket}\nor {:halt, socket}. The default implementation returns:\n\n    {:halt, push_redirect(socket, to: socket.view.fallback_path())}","ref":"Permit.Phoenix.LiveView.AuthorizeHook.html#module-usage"},{"type":"macro","title":"Permit.Phoenix.LiveView.AuthorizeHook.live_view_assign/3","doc":"","ref":"Permit.Phoenix.LiveView.AuthorizeHook.html#live_view_assign/3"},{"type":"function","title":"Permit.Phoenix.LiveView.AuthorizeHook.on_mount/4","doc":"","ref":"Permit.Phoenix.LiveView.AuthorizeHook.html#on_mount/4"},{"type":"module","title":"Permit.Phoenix.Plug","doc":"Authorization plug for the web application.\n\nIt automatically infers what CRUD action is represented by the currently executed controller\naction, and delegates to Permit to determine whether the action is authorized based\non current user's role.\n\nCurrent user is always fetched from `conn.assigns[:current_user]`, and their role is taken\nfrom the module specified in the app's `use Permit` directive. For instance if it's\nconfigured as follows:\n\n```\n# Authorization configuration module\ndefmodule MyApp.Authorization do\n  use Permit,\n    repo: Lvauth.Repo,\n    permissions_module: MyApp.Authorization.Permissions\nend\n\n# Permissions module - just as an example\ndefmodule MyApp.Authorization.Permissions do\n  use Permit.Permissions\n\n  def can(%{role: :manager} = role) do\n    # A :manager can do all CRUD actions on RouteTemplate, and can do :read on User\n    # if the User has public: true OR the User has :overseer_id equal to current user's\n    # id.\n    permit()\n    |> all(Lvauth.Planning.RouteTemplate)\n    |> read(Lvauth.Accounts.User, public: true)\n    |> read(LvMainFrame.Accounts.User,\n            fn user, other_user -> other_user.overseer_id == user.id end)\n  end\nend\n```\n\nThen controller can be configured the following way:\n\n```\ndefmodule LvauthWeb.Planning.RouteTemplateController do\n  plug Permit.Phoenix.Plug,\n    authorization_module: MyApp.Authorization,\n    loader: fn id -> Lvauth.Repo.get(Customer, id) end,\n    resource_module: Lvauth.Management.Customer,\n    id_param_name: \"id\",\n    except: [:example],\n    fetch_subject: fn conn -> conn.assigns[:signed_in_user] end\n    handle_unauthorized: fn conn -> redirect(conn, to: \"/foo\") end\n\n  def show(conn, params) do\n    # 1. If assigns[:current_user] is present, the \"id\" param will be used to call\n    #    Repo.get(Customer, params[\"id\"]).\n    # 2. can(role) |> read?(record) will be called on the loaded record and each user role.\n    # 3. If authorization succeeds, the record will be stored in assigns[:loaded_resources].\n    # 4. If any of the steps described above fails, the pipeline will be halted.\n  end\n\n  def index(conn, params) do\n    # 1. If assigns[:current_user] is present, can(role) |> read?(Customer) will be called on each role\n    # 2. If authorization succeeds, nothing happens.\n    # 3. If any of the steps described above fails, the pipeline will be halted.\n  end\n\n  def details(conn, params) do\n    # Behaves identically to :show because in :action_crud_mapping it's defined as :read.\n  end\n\n  def clone(conn, params) do\n    # 1. If assigns[:current_user] is present, the \"id\" param will be used to call\n    #    Repo.get(Customer, params[\"id\"]).\n    # 2. can(role) |> update?(record) will be called on the loaded record (as configured in :action_crud_mapping) and each role of user\n    # 3. If authorization succeeds, the record will be stored in assigns[:loaded_resources].\n    # 4. If any of the steps described above fails, the pipeline will be halted.\n  end\nend\n```\n\n##","ref":"Permit.Phoenix.Plug.html"},{"type":"function","title":"Permit.Phoenix.Plug.call/2","doc":"","ref":"Permit.Phoenix.Plug.html#call/2"},{"type":"function","title":"Permit.Phoenix.Plug.init/1","doc":"","ref":"Permit.Phoenix.Plug.html#init/1"},{"type":"module","title":"Permit.Phoenix.Types","doc":"Defines types for usage with Permit in the context of Phoenix applications.","ref":"Permit.Phoenix.Types.html"},{"type":"type","title":"Permit.Phoenix.Types.action_list/0","doc":"","ref":"Permit.Phoenix.Types.html#t:action_list/0"},{"type":"type","title":"Permit.Phoenix.Types.conn/0","doc":"","ref":"Permit.Phoenix.Types.html#t:conn/0"},{"type":"type","title":"Permit.Phoenix.Types.error_msg/0","doc":"A default error message for the controller in case there's no authorization.","ref":"Permit.Phoenix.Types.html#t:error_msg/0"},{"type":"type","title":"Permit.Phoenix.Types.fallback_path/0","doc":"A default fallback path for the controller in case there's no authorization.","ref":"Permit.Phoenix.Types.html#t:fallback_path/0"},{"type":"type","title":"Permit.Phoenix.Types.handle_unauthorized/0","doc":"A default handler that is called in case there is no authorization.","ref":"Permit.Phoenix.Types.html#t:handle_unauthorized/0"},{"type":"type","title":"Permit.Phoenix.Types.hook_outcome/0","doc":"","ref":"Permit.Phoenix.Types.html#t:hook_outcome/0"},{"type":"type","title":"Permit.Phoenix.Types.id_param_name/0","doc":"A name of a resource struct's ID parameter in controller or LiveView params, typically represented by a string.","ref":"Permit.Phoenix.Types.html#t:id_param_name/0"},{"type":"type","title":"Permit.Phoenix.Types.live_authorization_result/0","doc":"","ref":"Permit.Phoenix.Types.html#t:live_authorization_result/0"},{"type":"type","title":"Permit.Phoenix.Types.plug_opts/0","doc":"- `:authorization_module` -- (Required) The app's authorization module that uses `use Permit`.\n- `preload_actions` -- (Optional) The list of actions that resources will be preloaded and authorized in, in addition to :show, :delete, :edit and :update.\n- `loader` -- (Required, unless :repo defined) The loader, 1-arity function, used to fetch records in singular resource functions (:show, :edit, :update, :delete and other defined as :preload_actions). It is convenient to use context getter functions as loaders.\n- `resource` -- (Required) The struct module defining the specific resource the controller is dealing with.\n- `id_param_name` -- (Required, if singular record actions are present) The parameter name used to look for IDs of resources, passed to the loader function or the repo.\n- `fallback_path` -- (Optional) A string denoting redirect path when unauthorized. Defaults to \"/\".\n- `error_msg` -- (Optional) An error message to put into the flash when unauthorizd. Defaults to \"You do not have permission to perform this action.\"\n- `handle_unauthorized - (Optional) A function taking (conn), performing specific action when authorization is not successful. Defaults to redirecting to :fallback_path.","ref":"Permit.Phoenix.Types.html#t:plug_opts/0"},{"type":"type","title":"Permit.Phoenix.Types.socket/0","doc":"","ref":"Permit.Phoenix.Types.html#t:socket/0"},{"type":"exception","title":"Permit.Phoenix.RecordNotFoundError","doc":"","ref":"Permit.Phoenix.RecordNotFoundError.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.1"}}